if select(6, GetAddOnInfo("PitBull4_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local PitBull4 = _G.PitBull4
if not PitBull4 then
	error("PitBull4_VisualHelpers requires PitBull4")
end

-- :::::::::::: LOCALS ::::::::::::: -------------------------------------------------------------------------

local BASE_TEXTURE_PATH = [[Interface\AddOns\]] .. debugstack():match("[d%.][d%.][O%.]ns\\(.-)\\[A-Za-z]-%.lua") .. [[\]]

local L = PitBull4.L

local GetLayoutDB = PitBull4.Options.GetLayoutDB

local PitBull4_VisualHelpers = PitBull4:NewModule("VisualHelpers", "AceEvent-3.0", "AceTimer-3.0")
PitBull4_VisualHelpers:SetModuleType("custom")
PitBull4_VisualHelpers:SetName(L["Visual helpers"])
PitBull4_VisualHelpers:SetDescription(L["Show visual helpers for aura groups affecting a unit."])
PitBull4_VisualHelpers:SetDefaults({},{})

local LibSharedMedia = LibStub("LibSharedMedia-3.0", true)
LoadAddOn("AceGUI-3.0-SharedMediaWidgets")
local AceGUI = LibStub("AceGUI-3.0")

PitBull4_VisualHelpers.VH_INDICATORS = {}
local updated_guids = {}
local auras = {}

-- timer for OnUpdate execution control
local timerFrame = PitBull4.Controls.MakeFrame()
timerFrame:Hide()
local timer = 0
timerFrame:SetScript("OnUpdate",function(self, elapsed)
	timer = timer + elapsed
	if timer >= 0.2 then
		PitBull4_VisualHelpers:OnUpdate()
		timer = 0
	end
end)
local timer = 0

local helpers = PitBull4_VisualHelpers.db.profile.global.helpers

-- Fills an array of arrays with the information about the auras
local function reset_auras()
   for n in pairs(auras) do
      auras[n].present = false
      auras[n].caster = ''
   end
end
local function update_aura_list(unit)
   reset_auras()
	if not unit then return end
	local aura
	local caster
	local index = 1

	-- Loop through the auras
	while true do
		aura, _, _, _, _, _, _, caster = UnitAura(unit, index)
		if not aura then
			break
		end
		if auras[aura] then
		   auras[aura].present = true
		   auras[aura].caster = ''
		end
	   index = index + 1
	end
end
local function update_frame(frame)
   update_aura_list(frame.unit)
   local visible
   for helper_name, helper in pairs(helpers) do
      local indicator = PitBull4_VisualHelpers.VH_INDICATORS[helper_name]
      local db = indicator:GetLayoutDB(frame)
      if db.enabled then
         visible = false
         for _, aura in ipairs(helper.auras) do
            if auras[aura].present and (not db.only_self or auras[aura].caster=='player') then 
               visible = true
            end
         end
         if frame[indicator.full_name..'_show'] ~= visible then
            frame[indicator.full_name..'_show'] = visible
   		   indicator:Update(frame)
         end
      end
      indicator = nil
      db = nil
   end
end

-- :::::::::::: PitBull4_VisualHelpers ::::::::::::: -------------------------------------------------------------------------
function PitBull4_VisualHelpers:OnEnable()
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
end
                                
function PitBull4_VisualHelpers:PLAYER_ENTERING_WORLD()
   if not (PitBull4_VisualHelpers.db.profile.global.helpers 
           and next(PitBull4_VisualHelpers.db.profile.global.helpers)) then
      PitBull4_VisualHelpers:CreateDefaultGlobalHelpers()
      helpers = PitBull4_VisualHelpers.db.profile.global.helpers
   end
   self:UpdateIndicators()
	timerFrame:Show()
end

function PitBull4_VisualHelpers:UNIT_AURA(event, unit)
   -- TO CHECK
   -- for some reason when Beacon of Light its on this event keep
   -- firing over and over every 1 second even if there are no 
   -- changes in auras
	updated_guids[UnitGUID(unit)] = true
end
                         
function PitBull4_VisualHelpers:OnUpdate()
	if next(updated_guids) then
		for frame in PitBull4:IterateFrames() do
			if updated_guids[frame.guid] then
			   update_frame(frame)
			end
		end
	end
	wipe(updated_guids)
end

function PitBull4_VisualHelpers:OnDisable()
	timerFrame:Hide()
	wipe(self.VH_INDICATORS)
end

function PitBull4_VisualHelpers:UpdateIndicators()
   auras = {}
   for n, h in pairs(helpers) do
      if not PitBull4.modules["VisualHelpers: "..n] then
         self.VH_INDICATORS[n] = self:CreateVisualHelperIndicator(n)
      end
      for _, a in ipairs(h.auras) do
         if not auras[a] then
            auras[a] = {}
            auras[a].present = false
            auras[a].caster = ''
         end
      end
   end
	for frame in PitBull4:IterateFrames() do
	   update_frame(frame)
	end
end
